@@ -26,11 +26,12 @@ applylambda6 <- function(M, callback = function(){}){
   return(lambda6)
 }
 
-applyomegaPFA <- function(m, callback = function(){}){
+applyomegaPFA <- function(m, callback = function(){}, loadings = FALSE){
 
   f <- try(pfaArma(m), silent = TRUE)
   if (inherits(f, "try-error")) {
     om <- NaN
+    l_fa <- NaN
     warning("singular bootstrapped covariance matrices encountered when computing omega")
   } else {
     l_fa <- f$loadings
@@ -42,10 +43,22 @@ applyomegaPFA <- function(m, callback = function(){}){
 
   callback()
 
-  return(om)
+  if (loadings) {
+    return(list(om = om, loadings = l_fa))
+  } else {
+    return(om)
+  }
 }
 
 # old functions without Cpp
+
+applyGlbNoCpp <- function(M, callback = function(){}){
+  glbObj <- psych::glb.fa(M)
+  glb <- glbObj$glb
+  callback()
+  return(glb)
+}
+
 applyalphaNoCpp <- function(M, callback = function(){}){
   p <- ncol(M)
   a <- (p / (p - 1)) * (1 - (sum(diag((M))) / sum(M)))
@@ -62,11 +75,12 @@ applylambda2NoCpp <- function(M, callback = function(){}){
 }
 
 applylambda4NoCpp <- function(M, callback = function(){}){
-  if (ncol(M) < 15)
+  if (ncol(M) < 15) {
     out <- MaxSplitExhaustive(M)
-  else
+  } else {
     l4 <- quant.lambda4(M)
-  out <- quantile(l4, prob = 1)
+    out <- quantile(l4, prob = 1)
+  }
   callback()
   return(out)
 }
