@@ -128,7 +128,7 @@ lavMultiFileSeco <- function(k, ns) {
 }
 
 # generate model file for lavaan
-lavMultiFileSecoSyntax <- function(k, ns, model, colnams, model_opts) {
+lavMultiFileSecoSyntax <- function(k, ns, model, colnams) {
 
   beta_names <- paste("gl", 1:ns, sep = "")
 
@@ -139,8 +139,8 @@ lavMultiFileSecoSyntax <- function(k, ns, model, colnams, model_opts) {
     ns <- mod_out$mod_n.factors
   }
 
-  imat <- model_opts$imat
-  idex <- model_opts$idex
+  imat <- matrix(FALSE, k, ns)
+  idex <- list()
   lambda_names <- list()
   names <- list()
   for (i in seq_len(length(mod))) {
@@ -446,27 +446,6 @@ sum_X_Xt <- function(X) {
 }
 
 
-# convert a index matrix to a lavaan syntax -> helps with simulations
-# model file for seco model with crossloadings
-# also works for the correlated model
-lavMultiFileCross <- function(imat) {
-  k <- nrow(imat)
-  ns <- ncol(imat)
-  names <- 0
-  for(i in 1:k){
-    names[i] <- paste0("x", i)
-  }
-
-  str <- list()
-  mod <- NULL
-  for (i in 1:ns) {
-    str[[i]] <- paste0(names[imat[,i]], collapse = " + ")
-    mod <- paste0(mod, "s", i, " =~ ", str[[i]], " \n ")
-  }
-
-  return(out <- list(names = names, model = mod))
-}
-
 omegaCorr <- function(ll, phi, tt) {
   return(sum(colSums(ll) * colSums(ll) %*% phi) / (sum(colSums(ll) * colSums(ll) %*% phi) + sum(tt)))
 }
@@ -475,43 +454,44 @@ implCovCorr <- function(lambda, phi, theta) {
   return(lambda %*% phi %*% t(lambda) + theta)
 }
 
-
-# generate lav model file from no input
-
+# from ChatGpt
 lavMultiFileCorr <- function(k, ns) {
+  if (k %% ns != 0) stop("k must be divisible by ns")
 
-  lambda_names <- matrix(paste("sl", 1:k, sep = ""), k/ns, ns)
-  names <- matrix(paste("x", 1:k, sep = ""), k/ns, ns)
-  theta_names <- paste("e", 1:k, sep = "")
+  lambda_names <- matrix(paste0("sl", 1:k), k / ns, ns)
+  item_names <- matrix(paste0("x", 1:k), k / ns, ns)
+  theta_names <- paste0("e", 1:k)
 
-  mod <- NULL
   ss <- paste0("s", 1:ns)
+
+  # Factor loadings
+  mod <- NULL
   for (i in 1:ns) {
-    mod <- paste0(mod, ss[i], " =~ ", paste(paste(lambda_names[, i], "*", names[, i], sep = ""),
-                                                                   collapse = " + "), "\n")
+    mod <- paste0(mod,
+                  ss[i], " =~ ",
+                  paste(paste0(lambda_names[, i], "*", item_names[, i]), collapse = " + "),
+                  "\n")
   }
 
-  ss_comb <- combn(ss, m = 2)
-  rho_names <- paste("rl", 1:ncol(ss_comb), sep = "")
-  rhos <- NULL
-  vars <- paste0(ss, " ~~ ", "1*", ss, "\n", collapse = "")
-  for (i in 1:ncol(ss_comb)) {
-    rhos <- paste0(rhos, ss_comb[1, i], " ~~ ", paste0(rho_names[i], "*", ss_comb[2, i]), "\n")
-  }
+  # Factor correlations
+  ss_comb <- combn(ss, 2)
+  rho_names <- paste0("rl", 1:ncol(ss_comb))
+  cor_lines <- paste0(ss_comb[1, ], " ~~ ", rho_names, "*", ss_comb[2, ], collapse = "\n")
+  var_lines <- paste0(ss, " ~~ 1*", ss, collapse = "\n")
+  mod <- paste0(mod, "\n", cor_lines, "\n", var_lines, "\n")
 
-  mod <- paste0(mod, "\n", rhos, "\n", vars, "\n")
+  # Residual variances
+  mod <- paste0(mod,
+                paste(paste(c(item_names), " ~~ ", theta_names, "*", c(item_names)),
+                      collapse = "\n"),
+                "\n")
 
-  # correlation matrix for latents:
+  # Omega total calculation
   cor_mat <- matrix(NA, ns, ns)
   diag(cor_mat) <- 1
   cor_mat[lower.tri(cor_mat)] <- rho_names
   cor_mat[upper.tri(cor_mat)] <- t(cor_mat)[upper.tri(cor_mat)]
 
-  # extra stuff for omega calc
-  mod <- paste0(mod, paste(paste(c(names), " ~~ ", theta_names, "*",
-                                 c(names), sep = ""), collapse = "\n"), "\n")
-
-  # define extra parameters
   sl <- NULL
   tp <- NULL
   for (i in 1:ns) {
@@ -521,68 +501,66 @@ lavMultiFileCorr <- function(k, ns) {
   }
 
   sum_sl <- paste0("spec_loading := ", paste0("sm", 1:ns, collapse = " + "), "\n")
-  sum_errs <- paste("residual_var :=", paste(theta_names, collapse = " + "), "\n")
-  omega_t <- "omega_t := (spec_loading) / (spec_loading + residual_var) \n"
-  mod <- paste0(mod, sl, tp, sum_sl, sum_errs, omega_t)
+  sum_errs <- paste0("residual_var := ", paste(theta_names, collapse = " + "), "\n")
+  omega_t <- "omega_t := (spec_loading) / (spec_loading + residual_var)\n"
 
-  return(list(names = names, model = mod))
+  model_out <- paste0(mod, sl, tp, sum_sl, sum_errs, omega_t)
+
+  return(list(names = item_names, model = model_out))
 }
 
 
-# generate model file for lavaan from input model file
-lavMultiFileCorrSyntax <- function(k, ns, model, colnams, model_opts) {
-
-  mod_out <- model_opts$mod_out
+lavMultiFileCorrSyntax <- function(k, ns, model, colnams) {
+  mod_out <- modelSyntaxExtract(model, colnams)
   mod <- mod_out$mod
-
-  if (ns != model_opts$mod_out$mod_n.factors) {
+  if (ns != mod_out$mod_n.factors) {
     warning("n.factors is unequal to specified factors in model syntax")
-    ns <- model_opts$mod_out$mod_n.factors
+    ns <- mod_out$mod_n.factors
   }
 
-  imat <- model_opts$imat
-  idex <- model_opts$idex
+  ss <- mod_out$fac_names
+  imat <- matrix(FALSE, k, ns)
+  idex <- list()
   lambda_names <- list()
   names <- list()
-  for (i in seq_len(length(mod))) {
-    tmp_mod <- unlist(strsplit(mod[[i]], " "))
+  for (i in seq_along(mod)) {
+    tmp_mod <- unlist(strsplit(mod[[i]], "\\s+"))
     idex[[i]] <- which(colnams %in% tmp_mod)
     imat[idex[[i]], i] <- TRUE
-    lambda_names[[i]] <- paste0("l", i, seq_len(length(idex[[i]])))
+    lambda_names[[i]] <- paste0("l", i, seq_along(idex[[i]]))
     names[[i]] <- tmp_mod[nchar(tmp_mod) > 0]
   }
 
-  theta_names <- paste("e", 1:k, sep = "")
+  theta_names <- paste0("e", 1:k)
 
+  # Factor loadings
   modfile <- NULL
   for (i in 1:ns) {
-    modfile <- paste0(modfile, mod_out$fac_names[i], " =~ ", paste(paste(lambda_names[[i]], "*", names[[i]], sep = ""),
-                                                                   collapse = " + "), "\n")
+    modfile <- paste0(modfile,
+                      ss[i], " =~ ",
+                      paste(paste0(lambda_names[[i]], "*", names[[i]]), collapse = " + "),
+                      "\n")
   }
 
-  vars <- mod_out$fac_names
-  vars <- paste0(vars, " ~~ ", "1*", vars, "\n", collapse = "")
-  ss <- mod_out$fac_names
-  ss_comb <- combn(ss, m = 2)
-  rho_names <- paste("rl", 1:ncol(ss_comb), sep = "")
+  # Factor correlations
+  ss_comb <- combn(ss, 2)
+  rho_names <- paste0("rl", 1:ncol(ss_comb))
+  cor_lines <- paste0(ss_comb[1, ], " ~~ ", rho_names, "*", ss_comb[2, ], collapse = "\n")
+  var_lines <- paste0(ss, " ~~ 1*", ss, collapse = "\n")
+  modfile <- paste0(modfile, "\n", cor_lines, "\n", var_lines, "\n")
 
-  rhos <- NULL
-  for (i in 1:ncol(ss_comb)) {
-    rhos <- paste0(rhos, ss_comb[1, i], " ~~ ", paste0(rho_names[i], "*", ss_comb[2, i]), "\n")
-  }
+  # Residual variances
+  modfile <- paste0(modfile,
+                    paste(paste(unique(unlist(names)), " ~~ ", theta_names, "*",
+                                unique(unlist(names))), collapse = "\n"),
+                    "\n")
 
-  modfile <- paste(modfile, "\n", rhos, "\n", vars, "\n")
-
-  # correlation matrix for latents:
+  # Omega total
   cor_mat <- matrix(NA, ns, ns)
   diag(cor_mat) <- 1
   cor_mat[lower.tri(cor_mat)] <- rho_names
   cor_mat[upper.tri(cor_mat)] <- t(cor_mat)[upper.tri(cor_mat)]
 
-  # extra stuff for omega calc
-  modfile <- paste0(modfile, paste(paste(unique(unlist(names)), " ~~ ", theta_names, "*",
-                                         unique(unlist(names)), sep = ""), collapse = "\n"), "\n")
-
   sl <- NULL
   tp <- NULL
   for (i in 1:ns) {
@@ -592,11 +570,10 @@ lavMultiFileCorrSyntax <- function(k, ns, model, colnams, model_opts) {
   }
 
   sum_sl <- paste0("spec_loading := ", paste0("sm", 1:ns, collapse = " + "), "\n")
-  sum_errs <- paste("residual_var :=", paste(theta_names, collapse = " + "), "\n")
-  omega_t <- "omega_t := (spec_loading) / (spec_loading + residual_var) \n"
+  sum_errs <- paste0("residual_var := ", paste(theta_names, collapse = " + "), "\n")
+  omega_t <- "omega_t := (spec_loading) / (spec_loading + residual_var)\n"
+
   model_out <- paste0(modfile, sl, tp, sum_sl, sum_errs, omega_t)
 
-  return(out <- list(model = model_out, factor_names = mod_out$fac_names))
+  return(list(model = model_out, factor_names = ss))
 }
-
-
