@@ -8,7 +8,8 @@
 #' @param estimate A character string indicating what estimate to plot from the strel output object
 #' @param low.bound A number for the threshold to be tested against
 #' @examples
-#' pStrel(strel(asrm, "lambda2", n.chains = 2, n.iter = 100, freq = FALSE), "lambda2", .80)
+#' pStrel(strel(asrm, "lambda2", n.chains = 2, n.iter = 150, freq = FALSE),
+#' "lambda2", .80)
 #' @export
 pStrel <- function(x, estimate, low.bound) {
 
@@ -36,45 +37,56 @@ pStrel <- function(x, estimate, low.bound) {
 }
 
 
-#' prior and posterior probability of omega_t and omega_h being bigger than thresholds
+#' prior and posterior probability of omega_t and omega_h being larger than thresholds
 #' @description
 #' takes mcmc posterior samples of omega_t and omega_h
 #' and calculates the prior and posterior probability of the estimate being bigger
 #' or smaller than an arbitrary value
 #'
-#' @param x A strel output object (list)
+#' @param x A bomegas output object (list)
 #' @param cutoff.t A number indicating the threshold for omega_t
 #' @param cutoff.h A number indicating the threshold for omega_h
 #'
 #' @examples
-#' pOmegas(bomegas(upps, n.factors = 5, n.chains = 2, n.iter = 100, n.burnin = 50,
-#' missing = "listwise"))
+#' pOmegas(bomegas(upps, n.factors = 5, n.chains = 2, n.iter = 150, n.burnin = 50,
+#' disableMcmcCheck = TRUE, missing = "listwise"))
 #' @export
 pOmegas <- function(x, cutoff.t = .80, cutoff.h = .60) {
 
+  if (x$model.type != "correlated") {
+    samph <- as.vector(x$omega_h$chains)
+    objh <- ecdf(samph)
+    post_prob_h <- 1 - objh(cutoff.h)
+
+  }
+
   sampt <- as.vector(x$omega_t$chains)
-  samph <- as.vector(x$omega_h$chains)
   objt <- ecdf(sampt)
-  objh <- ecdf(samph)
   post_prob_t <- 1 - objt(cutoff.t)
-  post_prob_h <- 1 - objh(cutoff.h)
 
   # prior prob
-  priors <- omegasPrior(x$k, x$n.factors, nsamp = 2e3, x$prior_params$a0, x$prior_params$b0,
-                        x$prior_params$l0, x$prior_params$A0,
-                        x$prior_params$c0, x$prior_params$d0,
-                        x$prior_params$beta0, x$prior_params$B0,
-                        x$prior_params$p0, x$prior_params$R0)
+  if (x$model.type == "second-order") {
+    priors <- omegasSecoPrior(x, nsamp = 2e3)
+  } else if (x$model.type == "bi-factor") {
+    priors <- omegasBifPrior(x, nsamp = 2e3)
+  } else if (x$model.type == "correlated") {
+    priors <- omegasCorrPrior(x, nsamp = 2e3)
+  }
 
   priort <- ecdf(priors$omt_prior)
-  priorh <- ecdf(priors$omh_prior)
-
   prior_prob_t <- 1 - priort(cutoff.t)
-  prior_prob_h <- 1 - priorh(cutoff.h)
 
-  out <- matrix(c(prior_prob_t, prior_prob_h, post_prob_t, post_prob_h, cutoff.t, cutoff.h),
-                3, 2, byrow = TRUE)
-  colnames(out) <- c("omega_t", "omega_h")
-  rownames(out) <- c("prior_prob", "posterior_prob", "cutoff")
+  if (x$model.type != "correlated") {
+    priorh <- ecdf(priors$omh_prior)
+    prior_prob_h <- 1 - priorh(cutoff.h)
+    out <- matrix(c(prior_prob_t, prior_prob_h, post_prob_t, post_prob_h),
+                  2, 2, byrow = TRUE)
+    colnames(out) <- c(paste0("omega_t>", cutoff.t), paste0("omega_h>", cutoff.h))
+  } else {
+    out <- matrix(c(prior_prob_t, post_prob_t),
+                  2, 1, byrow = TRUE)
+    colnames(out) <- c(paste0("omega_t>", cutoff.t))
+  }
+  rownames(out) <- c("prior_prob", "posterior_prob")
   return(out)
 }
